using CRMPluginUtils;
using MicrosoftOutlookPlugin.Properties;
using MicrosoftOutlookServiceProvider;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace MicrosoftOutlookPlugin
{
  public partial class OutlookPluginConfigurationControl : PluginConfigurationControl
  {
    private ConfigurationManager configurationManager = new ConfigurationManager("3CXCRMUser.ini");
    
    private void loadLabelsFromResources()
    {
      lblFolderListExplanation.Text = LocalizedResourceManager.GetString("DotNetScript", "OutlookPluginConfigurationControl.lblFolderListExplanation.Text");
      chkShowFolderSelectionDialog.Text = LocalizedResourceManager.GetString("DotNetScript", "OutlookPluginConfigurationControl.chkShowFolderSelectionDialog.Text");
      toolTipProvider.SetToolTip(infoPictureShowFolderSelection, LocalizedResourceManager.GetString("DotNetScript", "OutlookPluginConfigurationControl.chkShowFolderSelectionDialog.InfoText"));
      
      infoPictureShowFolderSelection.Location = new Point(6 + chkShowFolderSelectionDialog.Location.X + chkShowFolderSelectionDialog.Width, infoPictureShowFolderSelection.Location.Y);
    }
    
    private void loadInitialValues()
    {
      try
      {
        disableNotifications();
        chkShowFolderSelectionDialog.Checked = configurationManager.GetValue("Microsoft Outlook Add-in", "ShowFolderSelectionDialog", "False") == "True";
      }
      finally
      {
        enableNotifications();
      }
    }

    public OutlookPluginConfigurationControl()
    {
      InitializeComponent();

      loadLabelsFromResources();
      loadInitialValues();
      OutlookHelper.RegisterAddin();
    }

    private void OutlookPluginConfigurationControl_Load(object sender, EventArgs e)
    {
      refreshButton.PerformClick();
    }

    public override Image GetPluginIcon()
    {
      return Resources.OutlookPlugin;
    }

    public override string GetPluginName()
    {
      return "Microsoft Outlook";
    }

    public override void LoadConfiguration()
    {
      loadInitialValues();
    }

    public override void Save()
    {
      configurationManager.SetValue("Microsoft Outlook Add-in", "ShowFolderSelectionDialog", chkShowFolderSelectionDialog.Checked ? "True" : "False");

      int enabledIndex = 0;
      for (int index = 0; index < contactFolderListBox.Items.Count; ++index)
      {
        if (contactFolderListBox.GetItemChecked(index))
        {
          configurationManager.SetValue("Microsoft Outlook Plugin", "EnabledContactFolder_" + enabledIndex.ToString(), contactFolderListBox.Items[index].ToString());
          ++enabledIndex;
        }
      }

      configurationManager.SetValue("Microsoft Outlook Plugin", "IsEveryContactFolderEnabled", "False");
      configurationManager.SetValue("Microsoft Outlook Plugin", "EnabledContactFolderCount", contactFolderListBox.CheckedItems.Count.ToString());
    }
    
    private void chkShowFolderSelectionDialog_CheckedChanged(object sender, EventArgs e)
    {
      notifyControlChanged();
    }
    
    private void contactFolderListBox_SelectedIndexChanged(object sender, EventArgs e)
    {
      moveUpButton.Enabled = contactFolderListBox.SelectedIndex > 0;
      moveDownButton.Enabled = contactFolderListBox.SelectedIndex >= 0 && contactFolderListBox.SelectedIndex < contactFolderListBox.Items.Count - 1;
      notifyControlChanged();
    }

    private void moveUpButton_Click(object sender, EventArgs e)
    {
      int selectedIndex = contactFolderListBox.SelectedIndex;
      string contactFolderPath = contactFolderListBox.Items[selectedIndex].ToString();
      bool isChecked = contactFolderListBox.GetItemChecked(selectedIndex);
      contactFolderListBox.Items.RemoveAt(selectedIndex);
      contactFolderListBox.Items.Insert(selectedIndex - 1, contactFolderPath);
      contactFolderListBox.SetItemChecked(selectedIndex - 1, isChecked);
      contactFolderListBox.SelectedIndex = selectedIndex - 1;
      notifyControlChanged();
    }

    private void moveDownButton_Click(object sender, EventArgs e)
    {
      int selectedIndex = contactFolderListBox.SelectedIndex;
      string contactFolderPath = contactFolderListBox.Items[selectedIndex].ToString();
      bool isChecked = contactFolderListBox.GetItemChecked(selectedIndex);
      contactFolderListBox.Items.RemoveAt(selectedIndex);
      contactFolderListBox.Items.Insert(selectedIndex + 1, contactFolderPath);
      contactFolderListBox.SetItemChecked(selectedIndex + 1, isChecked);
      contactFolderListBox.SelectedIndex = selectedIndex + 1;
      notifyControlChanged();
    }
    
    private void refreshButton_Click(object sender, EventArgs e)
    {
      try
      {
        Cursor = Cursors.WaitCursor;
        
        if (configurationManager.GetValue("Microsoft Outlook Add-in", "IsRunning", "False") != "True")
          throw new ApplicationException(LocalizedResourceManager.GetString("DotNetScript", "OutlookPluginConfigurationControl.MessageBox.Error.OutlookAddinIsNotRunning"));
        
        IOutlookServiceProvider serviceProvider = OutlookHelper.GetServiceProvider();
        OutlookHelper.RefreshContactFolderList(serviceProvider);
        List<ContactFolder> contactFolderList = OutlookHelper.GetContactFolderList();
        List<ContactFolder> enabledContactFolderList = OutlookHelper.GetEnabledContactFolderList();

        contactFolderListBox.Items.Clear();
        foreach (ContactFolder enabledContactFolder in enabledContactFolderList)
          contactFolderListBox.Items.Add(enabledContactFolder.Path, true);

        foreach (ContactFolder contactFolder in contactFolderList)
        {
          if (!enabledContactFolderList.Contains(contactFolder))
            contactFolderListBox.Items.Add(contactFolder.Path, false);
        }

        if (contactFolderListBox.Items.Count > 0)
          contactFolderListBox.SelectedIndex = 0;

        notifyControlChanged();
      }
      catch (Exception exc)
      {
        MessageBox.Show(String.Format(LocalizedResourceManager.GetString("DotNetScript", "OutlookPluginConfigurationControl.MessageBox.Error.GettingContactFolderList"), exc.Message), "3CX Outlook Plug-in", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }
      finally
      {
        Cursor = Cursors.Default;
      }
    }
  }
}
